(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON-Web.
 *
 * UMLMON-Web is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON-Web is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON-Web; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: startpage.ml 268 2010-08-02 00:33:37Z gerd $
 * ----------------------------------------------------------------------
 *
 *)
 
open Wd_dialog
open Wd_run_cgi
open Wd_types
open Netcgi
open Definitions
 
open Umlmonctrl_aux

module Mon = Umlmonctrl_clnt.Ctrl.V1
module Dir = Umlmonctrl_clnt.Directory.V1


let ws_re = Str.regexp "[ \t\r\n]+"

let at_id_re = Str.regexp "^\\([^@]+\\)@\\(.*\\)$"

let split_id id =
  if Str.string_match at_id_re id 0 then
    ( Str.matched_group 1 id, Str.matched_group 2 id )
  else
    ( id, "localhost")
;;


class startpage universe name env =
object (self)
  inherit dialog universe name env
  inherit std_behaviour


  method private try_prepare_page() =
    if self # string_variable "id" = "" && self # page_name = "enter-password" then
      self # set_next_page "home";
    if self # page_name = "home" || self # next_page = "home" then (
      let hosts_ent, _ =
	self # application # dtd # gen_entity "hosts" in
      let hosts_str =
	Pxp_dtd.Entity.replacement_text hosts_ent in
      let hosts = Str.split ws_re hosts_str in
      let creds = self # creds in
      let id_list = Common.get_monitors_in_parallel hosts in
      let status_list = Common.get_status_in_parallel id_list creds in
      let monitors =
	List.map2
	  (fun id status ->
	     let (local_id, host) = split_id id in
	     (id, host, local_id, status)
	  )
	  id_list
	  status_list in
(*
      let ues = Unixqueue.create_unix_event_system() in
      let monitors =
	List.flatten
	  (List.map
	     (fun host ->
		try
		  let client = Umldir_client.create_client ues host in
		  let l = Array.to_list (Dir.get_monitors client ()) in
		  Rpc_client.shut_down client;
		  let l' = List.map (fun mon -> 
				       let id = mon.monitor_id ^ "@" ^ host in
				       let status = Common.get_status id creds in
				       (id,
					host, 
					mon.monitor_id,
					status
				       )
				    ) l in
		  l'
		with
		  | Unix.Unix_error(Unix.ECONNREFUSED,_,_) ->
		      []
	     )
	     hosts) in
 *)
      let monitors' =
	List.sort (fun (_, _, id1, _) (_, _, id2, _) -> compare id1 id2) monitors in
      let vm_list_name =
	Alist_value (List.map (fun (id, _, name, _) -> (id, String_value name)) monitors') in
      let vm_list_host =
	Alist_value (List.map (fun (id, host, _, _) -> (id, String_value host)) monitors') in
      let vm_list_status =
	Alist_value (List.map (fun (id, _,  _, s) -> (id, String_value s)) monitors') in
      self # set_variable "vm_list_name" vm_list_name;
      self # set_variable "vm_list_host" vm_list_host;
      self # set_variable "vm_list_status" vm_list_status;
    )

  method private try_handle() =
    match self # event with
      | Indexed_button("vm", id) ->
	  (* Check if we can log in: *)
	  ( try
	      Common.check_auth id self#creds;
	      let vm_dlg = !Registry.new_vm universe env id in
	      vm_dlg # set_variable "password" (self # variable "password");
	      call_dialog self vm_dlg
	    with
	      | Common.Wrong_password ->
		  self # set_variable "id" (String_value id);
		  raise (Change_page "enter-password")
	  )
      | Button "login" ->
	  (* Check if we can log in: *)
	  let id = self # string_variable "id" in
	  ( try
	      Common.check_auth id self#creds;
	      self # set_variable "id" (String_value "");
	      let vm_dlg = !Registry.new_vm universe env id in
	      vm_dlg # set_variable "password" (self # variable "password");
	      call_dialog self vm_dlg
	    with
	      | Common.Wrong_password ->
		  raise (Change_page "enter-password")
	  )
      | _ ->
	  ()

  method private check_auth () = ()
end
;;


Registry.new_startpage :=
  fun universe env ->
    let dlg = universe # create env "startpage" in
    dlg
;;
